#!/usr/bin/perl

#
# vdradmin.pl by Thomas Koch <tom@linvdr.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Or, point your browser to http://www.gnu.org/copyleft/gpl.html
#
# 08.10.2001
#

my($BASENAME);
BEGIN {
  $0 =~ /(^.*\/)/;
  $BASENAME = $1;
  unshift(@INC, $BASENAME . "lib/");
  unshift(@INC, $BASENAME);
}


use CGI qw/:no_debug/;
use IO::Socket;
use HTML::Template();
use Time::Local qw(timelocal);
use POSIX qw(strftime);
use MIME::Base64();

use strict;

my %CONFIG;
$CONFIG{VDRHOST}          = "video";
$CONFIG{SERVERPORT}       = 8001;
$CONFIG{VDRPORT}          = 2001;
$CONFIG{LANG}             = "Deutsch";
$CONFIG{CACHE_TIMEOUT}    = 360;
$CONFIG{PCHANL}           = 0;
$CONFIG{USER}             = "linvdr";
$CONFIG{PASS}             = "linvdr";
$CONFIG{LOGGING}          = 0;
$CONFIG{LOGFILE}          = "vdradmind.log";
$CONFIG{ZPUFA}            = 15;
$CONFIG{ZPUFE}            = 15;
$CONFIG{PCHAN_PL}         = 0;
$CONFIG{LOCALLASTGET}     = 0;
$CONFIG{AT_TIMEOUT}       = 180; 
$CONFIG{AT_FUNC}          = 0;
$CONFIG{AT_LIFETIME}      = 50;
$CONFIG{AT_PRIORITY}      = 50;
$CONFIG{LIFETIME}         = 99;
$CONFIG{PRIORITY}         = 99;


my $VERSION               = "0.15";
my $SERVERVERSION         = "vdradmind/$VERSION";
my $TEMPLATEDIR           = "${BASENAME}template";
my $CONFFILE              = "${BASENAME}vdradmind.conf";
my $LOGFILE               = "${BASENAME}$CONFIG{LOGFILE}";
my $PIDFILE               = "${BASENAME}vdradmind.pid";
my $AT_FILENAME           = "${BASENAME}vdradmind.at";
my $I18NFILE		  = "i18n.pl";

my($DEBUG) = 0;
my(%EPG, @CHAN, $SOCKET, $q); 
my(%mimehash) = ( 
  html => "text/html",
  png  => "image/png",
  gif  => "image/gif",
  jpg  => "image/jpeg"
);

my($DAEMON) = 1;
for(@ARGV) {
  if(/-h|--help/) {
    print("Usage $0 [OPTION]...\n");
    print("A perl client for the Linux Video Disk Recorder.\n\n");
    print("  -nf  --nofork   don't fork\n");
    print("  -c   --config   run configuration dialog\n");
    print("  -k   --kill     kill a fork'ed vdradmin\n");
    print("  -h   --help     this here\n");
    print("\nReport bugs to <tom\@linvdr.org>.\n");
    exit(0);
  }
  if(/--nofork|-nf/) { $DAEMON = 0; last; }
  if(/--config|-c/) {
    $CONFIG{VDRHOST} = 
      Question("What's your VDR hostname (e.g video.intra.net)?", "video");
    $CONFIG{VDRPORT} = 
      Question("What's the port VDR listen to SVDRP query's?", "2001");
    $CONFIG{SERVERPORT} = 
      Question("On which port should vdradmin answer?", "8001");
    $CONFIG{USER} = Question("Username?", "linvdr");
    $CONFIG{PASS} = Question("Password?", "linvdr");
    
    open(CONF, ">$CONFFILE") || die "Cannot open $CONFFILE: $!\n";
    for(keys(%CONFIG)) {
      print(CONF "$_ = $CONFIG{$_}\n");
    }
    close(CONF);

    print("Config file sucessfull written.\n");
  }
  if(/--kill|-k/) {
    open(CONF, $PIDFILE);
    my($pid);
    $pid = <CONF>;
    close(CONF);
    kill(2, $pid);
    exit(0);
  }
}

if($DAEMON) {
  my($pid) = fork;
  if($pid != 0) {
    print("vdradmind.pl $VERSION started with pid $pid.\n");
    open(F, ">${BASENAME}vdradmind.pid"); print(F $pid); close(F);
    exit(0);
  }
}


ReadConfig();
my($Socket) = IO::Socket::INET->new( 
  Proto => 'tcp',
  LocalPort => $CONFIG{SERVERPORT},
  Listen => SOMAXCONN,
  Reuse => 1
);				     
die("can't start server: $!\n") if (!$Socket);
$Socket->timeout($CONFIG{AT_TIMEOUT} * 60) if($CONFIG{AT_FUNC});
$CONFIG{LOCALLASTGET} = 0;



##
# Mainloop
##
my($Client, $MyURL);

$SIG{PIPE} = sub { my $signal = shift };

while(1) {
  $Client = $Socket->accept();
  if(!$Client) {
    UptoDate(1);
    next;
  }
  my $peer = $Client->peerhost;
  
  #
  my $Request;
  $Request = <$Client>;
  my $LogRequest = $Request;
  $Request =~ s/^[A-Z]+ //;
  $Request =~ s/ [hH][Tt][Tt][pP]\/[0-9]+\.*[0-9]+[\n\r\015\013]+$//g;
  my $URI = $Request;
  $LogRequest =~ s/\r\n//g;
  
  
  my($AuthUser, $AuthPass);
  do {
    $Request = <$Client>;
    $Request =~ s/\r\n//g; 
    if($Request =~ /Authorization: basic (.*)/i) {
      ($AuthUser, $AuthPass) = split(/\:/, MIME::Base64::decode_base64($1));
    }   
  } while($Request);
  
  if(!$AuthUser) {
    SendNoAuthHeader("401");
    close($Client);
    next;
  } else {
    if($CONFIG{USER} ne $AuthUser || $CONFIG{PASS} ne $AuthPass) {
      SendNoAuthHeader("401");
      close($Client);
      next;
    }
  }
  
  $MyURL = "./vdradmin.pl";
  if($URI =~ /\/vdradmin.pl\?(.*)/) {
    $q = CGI->new($1);    
    for($q->param("aktion")) {
      prog_detail()      if(/prog_detail/);
      prog_list()        if(/prog_list/);
      timer_list()       if(/timer_list/);
      at_timer_edit()    if(/at_timer_edit/);
      at_timer_new()     if(/at_timer_new/);
      at_timer_save()    if(/at_timer_save/);
      at_timer_delete()  if(/at_timer_delete/);
      timer_new_form()   if(/timer_new_form/);
      timer_add()	 if(/timer_add/);
      timer_delete()     if(/timer_delete/);
      prog_summary()     if(/prog_summary/);
      rec_list()         if(/rec_list/);
      rec_delete()	 if(/rec_delete/);
      rec_detail()	 if(/rec_detail/);
      show_top()	 if(/show_top/);
      conf_list()	 if(/conf_list/);
      prog_switch()      if(/prog_switch/);
      rc_show()          if(/rc_show/);
      rc_hitk()          if(/rc_hitk/);
    }
  } elsif($URI eq "/") {
    my $template = HTML::Template->new(
      filename => "$TEMPLATEDIR/$CONFIG{LANG}/index.html",
      die_on_bad_params => 0
    );
    SendHeader("200", "text/html", $template->output); 
  } else {
    SendFile($URI);
  }
  
  Log("Client: $peer : $LogRequest L:" . length($LogRequest));
  
  close($Client);
}

#############################################################################
#############################################################################
sub ReadConfig {    
  if(-e $CONFFILE) {
    open(CONF, $CONFFILE);
    while(<CONF>) {
      chomp;
      my($key, $value) = split(/ \= /, $_, 2);
      $CONFIG{$key} = $value;
    }
    close(CONF);
  } else {
    open(CONF, ">$CONFFILE");
    for(keys(%CONFIG)) {
      print(CONF "$_ = $CONFIG{$_}\n");
    }
    close(CONF);
    return(1);
  }
  return(0);
}   



##
#
##
sub Question {
  my($quest, $default) = @_;
  print("$quest [$default]: ");
  my($answer);
  $answer = <STDIN>;
  if($answer eq "\n") {
    return($default);
  } else {
    return($answer);
  }
}


##
#
##
sub Log {
  if($CONFIG{LOGGING}) {
    open(LOGFILE, ">>" . $LOGFILE);
    my($text) = @_;
    print LOGFILE (strftime("%d.%m %H:%M:%S", localtime(time)) . " $text\n");	
    close(LOGFILE);
  }
}


##
#
##
sub RemoveLeadingZero {
  my($str) = @_;
  while(substr($str, 0, 1) == 0) {
    $str = substr($str, 1, (length($str) - 1));
  }
  return($str);
}


##
#
##
sub GetChannelDesc {
  my(%hash);
  for(@CHAN) {
    $hash{$_->{id}} = $_->{name};
  }
  return(%hash);
}


##
#
##
sub GetChannelDescByNumber {
  my $number = shift;

  if($number) {
    for(@CHAN) {
      if($_->{number} == $number) {
	return($_->{name});
      }
    }
  } else { return(0); }
}


##
#
##
sub ReadFile {
  my $file = shift;
  return if(!$file);
  
  my $buf;
  open(I18N, $file) || HTMLError("Cannot open $file!");
  while(<I18N>) {
    $buf .= $_;
  }
  return($buf);
}


##
#
##
sub FullDay {
  my @I18N_Days;
  eval(ReadFile("$TEMPLATEDIR/$CONFIG{LANG}/$I18NFILE"));
  return(@I18N_Days[shift]);
}
 

##
#
##
sub GetChannelID {
  my($sid) = $_[0];
  for(@CHAN) { 
    if($_->{id} == $sid) {
      return($_->{number});
    }
  }
}


##
#
##
sub EURL {
  my($text) = @_;
  $text =~ s/([^0-9a-zA-Z])/sprintf("%%%2.2x", ord($1))/ge;
  return($text);
}


##
#
##
sub HTMLError {
  my($error) = @_;   
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/error.html");
  $template->param(error => $error);
  SendHeader("200", "text/html", $template->output);
  return;
}


##
#
##
sub GetHeaderTime {
  my(@weekdays) = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun");
  my(@months) = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", 
    "Sep", "Oct", "Nov", "Dec");	     
  my($sec,$min,$hour,$day,$month,$year,$wday,$yday,$isdst) = gmtime(time);
  return(sprintf "%s, %s %s %s  %02d:%02d:%02d GMT", $weekdays[$wday], 
      $day, $months[$month], $year, $hour, $min, $sec);
}


##
#
##
sub PrintToClient {
  my $str;
  for(@_) {
    $str .= $_;
  }
  return if(!defined($str));
  print($Client $str) if($Client);
}


##
#
##
sub SendHeader {
  my($Status, $ContentType, $Data) = @_;
  return if(!$Client);
  PrintToClient("HTTP/1.0 $Status\r\n");
  PrintToClient("Date: ", GetHeaderTime(), "\r\n");
  PrintToClient("Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n");
  PrintToClient("Server: $SERVERVERSION\r\n");
  PrintToClient("Content-Length: ", length($Data), "\r\n") if($Data);
  PrintToClient("Connection: close\r\n");
  PrintToClient("Content-type: $ContentType\r\n\r\n") if($ContentType);
  PrintToClient($Data) if($Data);
}


##
#
##
sub SendForwardingHeader {
  my($Location) = @_;
  PrintToClient("HTTP/1.0 302 Found\r\n");
  PrintToClient("Date: ", GetHeaderTime(), "\r\n");
  PrintToClient("Server: $SERVERVERSION\r\n");
  PrintToClient("Connection: close\r\n");
  PrintToClient("Location: $Location\r\n");
  PrintToClient("Content-type: text/html\r\n\r\n");
}


##
#
##
sub SendNoAuthHeader {
  my($Status) = @_;
  my $template = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/noauth.html");
  my $Data = $template->output;
  PrintToClient("HTTP/1.0 $Status\r\n");
  PrintToClient("Date: ", GetHeaderTime(), "\r\n");
  PrintToClient("Server: $SERVERVERSION\r\n");
  PrintToClient("WWW-Authenticate: Basic realm=\"Privat\"\r\n");
  PrintToClient("Content-Length: ", length($Data), "\r\n") if($Data);
  PrintToClient("Connection: close\r\n");
  PrintToClient("Content-type: text/html\r\n\r\n");
  PrintToClient($Data);
}


##
#
##
sub SendFile {
  sub FileError {
    my $template = HTML::Template->new(
      filename => "$TEMPLATEDIR/$CONFIG{LANG}/noperm.html");
    $template->param(
      title => $_[0],
      h1    => $_[1],
      error => $_[2],
    );
    SendHeader("$_[0] $_[1]", "text/html", $template->output);
  }
  my($File) = @_;
  my($buf, $temp);
  $File =~ s/^\///;
  if(-e $BASENAME . $File) {
    if(-r $BASENAME . $File) {
      open(F, $BASENAME . $File);
      while(<F>) {
	$buf .= $_;
      }
      close(F);
      $temp = $File;
      $temp =~ /([A-Za-z0-9]+)\.([A-Za-z0-9]+)/;
      if(!$mimehash{$2}) { die("can't find mime-type \'$2\'\n"); }
      SendHeader("200", $mimehash{$2}, $buf);
    } else {
      FileError("403", "Forbidden", "You don't have permission to" .
	" access /$File on this server.");
    }
  } else {
    FileError("404", "Not Found", "The requested URL /$File was not" .
      " found on this server.");	
  }
}


##
#
##
sub encode_ref {
  my($tmp) = $_[0]->url(-relative=>1,-query=>1);
  my(undef, $query) = split(/\?/, $tmp, 2);
  return(MIME::Base64::encode_base64($query));
}


##
#
##
sub decode_ref {
  return(MIME::Base64::decode_base64($_[0]));
}


##
#
##
sub FillInZero {
  my($str, $length) = @_;
  while(length($str) < $length) {
    $str = "0$str";
  }
  return($str);
}


##
#
##
sub ChanTree {
  undef(@CHAN);
  for(SendCMD(1, "lstc")) {
    chomp;
    my($id, $temp) = split(/ /, $_, 2);	
    my($name, $frequency, $polarization, $diseqc, $symbolrate, $vpid, $apid,
      $tpid, $ca, $number) = split(/\:/, $temp);
    push(@CHAN, {
      id           => $id,
      name         => $name,
      frequency    => $frequency,
      polarization => $polarization,
      diseqc       => $diseqc,
      symbolrate   => $symbolrate,
      vpid         => $vpid,
      apid         => $apid,
      tpid         => $tpid,
      ca           => $ca,
      number       => $number
    }); 
  }
}


##
#
##
sub EPGTree {
  my(@DATA);
  for(SendCMD(1, "lste")) { push(@DATA, $_); }
  my($i); my(@events); undef(%EPG); my($id) = 0; my $bc = 0;
  for($i = 0; $i <= $#DATA; $i++) {
    $_ = $DATA[$i];
    chomp;
    if(/^C ([^ ]+) *(.*)/) {
      $bc++;
      undef(@events);
      my($number, $progname) = ($1, $2);
      for(; $i <= $#DATA; $i++) {
	$_ = $DATA[$i];
	if(/^E (.*) (.*) (.*) (.*)/ || /^E (.*) (.*) (.*)/) {
	  my($time, $duration) = ($2, $3);
	  my($title, $subtitle, $description);
	  for(; $i <= $#DATA; $i++) {
	    $_ = $DATA[$i];
	    if(/^T (.*)/) { $title = $1 }
	    if(/^S (.*)/) { $subtitle = $1 }
	    if(/^D (.*)/) { $description = $1 }
	    if(/^e/) {
	      #
	      my($cnumber);
	      for(@CHAN) {
		if($_->{number} == $number) {
		  $cnumber = $_->{id};
		  last;
		}
	      }
	      #
	      push(@events, {
		progname    => $progname,
		number      => $number,
		time        => $time,
		duration    => $duration,
		title       => $title,
		subtitle    => $subtitle,
		description => $description,
		id          => $id,
		cnumber     => $cnumber
	      });
	      $id++;
	      last;
	    }
	  }
	} elsif(/^c/) {
	  my($last) = 0; my(@temp);
	  for(sort({ $a->{time} <=> $b->{time} } @events)) {
	    next if($last == $_->{time});
	    push(@temp, $_);
	    $last = $_->{time};
	  }
	  $EPG{$number} = [ @temp ];
	  last;
	}
      }
    }
  }
  Log("EPGTree: $id events, $bc broadcasters");
}


##
#
##
sub CloseSocket {
  SendLine($SOCKET, "quit\r\n");
  my $tmp;
  $tmp = <$SOCKET>;
  close($SOCKET); 
  undef($SOCKET);
}


##
#
##
sub SendLine {
  my($Socket, $cmd) = @_;
  print("SendLine: $cmd") if($DEBUG);
  my($result) = send($Socket, $cmd, 0);
  if($result != length($cmd)) {
    HTMLError("Error sending command to host $CONFIG{VDRHOST}\n");
  }
}


##
#
##
sub SendCMD {
  my($pc) = shift;
  my($cmd) = @_; 
  my($error) = 0;
  
  my($fc) = 0;
  if($SOCKET == undef) {
    $SOCKET = IO::Socket::INET->new(
      PeerAddr => $CONFIG{VDRHOST}, 
      PeerPort => $CONFIG{VDRPORT}, 
      Proto => 'tcp') || ($error = 1);
      $fc = 1;
  }
  if($error) {
      HTMLError(
	"can't connect to host \"$CONFIG{VDRHOST}\" port $CONFIG{VDRPORT}!");
      return(1);
  }
    
  # if its the first connection, read greeting...
  my($tmp);
  if($fc) {
    $tmp = <$SOCKET>;
  }

  print("SendCMD: $cmd") if($DEBUG);
  
  SendLine($SOCKET, "$cmd\r\n");
  my(@output); 
  my($first) = 1; my($last);
  while(<$SOCKET>) {
    if($first) { $last = substr($_, 0, 3); }
    s/\r//g;
    if(substr($_, 3, 1) eq "-") {
      s/^$last-//;
      push(@output, $_);
    } else {
      s/^$last //;
      push(@output, $_);
      last;
    }
  }
  
  if($DEBUG) {
    for(@output) {
      print "A: $_";
    }
  }
  
  #
  if(!$pc) {
    print("SendCMD: Closing connection.\n") if($DEBUG);
    CloseSocket();
  }
  
  return(@output);
}

##
#
##
sub UptoDate {
  if(((time - $CONFIG{LOCALLASTGET}) >= $CONFIG{CACHE_TIMEOUT}) || $_[0]) {
    ChanTree();
    EPGTree();
    AutoTimer();
    $CONFIG{LOCALLASTGET} = time;
    CloseSocket();
  }
  return(0);
}


##
#
##
sub AT_Read {
  my(@at);
  if(-e $AT_FILENAME) {
    open(AT_FILE, $AT_FILENAME) ||
      HTMLError("Cant open $AT_FILENAME!");
    while(<AT_FILE>) {
      chomp;
      next if($_ eq "");
      my($active, $pattern, $section, $start, $stop, $episode, $prio, $lft, $channel) = split(/\:/, $_);
      push(@at, {
	active   => $active,
	pattern  => $pattern,
	section  => $section,
	start    => $start,
	stop     => $stop,
	episode  => $episode,
	prio     => $prio,
	lft      => $lft,
	channel  => $channel
      });
    }
    close(AT_FILE);
  }
  return(@at);
}


##
#
##
sub AT_Write {
  my @at = @_;
  open(AT_FILE, ">" . $AT_FILENAME) || 
    HTMLError("Cant open $AT_FILENAME!");
  for(@at) {
    my $temp;
    for my $item (qw(active pattern section start stop episode prio lft channel)) {
      if(length($temp) == 0) {
	$temp = $_->{$item};
      } else {
	$temp .= ":" . $_->{$item};
      }
    }
    print AT_FILE $temp, "\n";
  }
  close(AT_FILE);
}


##
#
##
sub AutoTimer {
  return if(!$CONFIG{AT_FUNC});
  Log("Auto Timer: Scanning for events...");
  my($search, $start, $stop) = @_;
 
  my(@at) = AT_Read();
  
  for my $sender (keys(%EPG)) {
    for my $event (@{$EPG{$sender}}) {
      for my $at (@at) {
	
	if(!$at->{active}) {
	  next;
	}
	
	if($at->{channel}) {
	  if($at->{channel} != $event->{number}) {
	    next;
	  }
	}

	#
	my @section;
	if($at->{section} & 1) {
	  push(@section, "title");
	} elsif($at->{section} & 2) {
	  push(@section, "subtitle");
	} elsif($at->{section} & 4) {
	  push(@section, "description");
	}
	
	#
	next if(!$at->{pattern});
      
    	#
	my $fs = 0;
	for my $section (@section) {
	  my $fp = 1;
	  for my $pattern (split(/ +/, $at->{pattern})) {
	    if($event->{$section} !~ /$pattern/i) {
	      $fp = 0;
	    }
	  }
	  next if(!$fp);
	  $fs = 1;
	}	
	next if(!$fs);
	
	#
	if(defined($at->{start})) {
	
	  if(((strftime("%H%M", localtime($event->{time}))) < $at->{start})) {
	    if(strftime("%H%M", localtime($at->{stop})) > strftime("%H%M", localtime($at->{start}))) {
	      next;
	    }
	  }
	}

	#
	if(defined($at->{stop})) {
	  ($at->{stop} < $at->{start}) ? $at->{stop} += 2400 : undef;
	  if((strftime("%H%M", localtime($event->{time} + $event->{duration}))) > $at->{stop}) {
	    next;
	  }
	}
	
	#	print($event->{title} . " " . (strftime("%H:%M", localtime($event->{time}))) . " " . (strftime("%H:%M", localtime($event->{time} + $event->{duration}))) . "\n");
	
	if($at->{episode}) {
	  ProgTimer(1, $event->{cnumber}, $event->{time}, $event->{time} + 
	    $event->{duration}, $event->{title} . "~" . $event->{subtitle}, "", $at->{prio}, $at->{lft});
	} else {
	  ProgTimer(1, $event->{cnumber}, $event->{time}, $event->{time} + 
	    $event->{duration}, $event->{title}, "", $at->{prio}, $at->{lft});
	}
      }
    }
  }
  Log("Auto Timer: Done.");
}


##
#
##
sub ProgTimer {
  my($active, $cnumber, $start, $stop, $name, $summary, $prio, $lft) = @_;
  $name =~ s/\:/ /g;
  
  $start -= ($CONFIG{ZPUFA} * 60);
  $stop += ($CONFIG{ZPUFE} * 60);
  
  ($prio = $CONFIG{AT_PRIORITY}) if(!$prio);
  ($lft  = $CONFIG{AT_LIFETIME}) if(!$lft);

  my($newt) = "$active:$cnumber:" . 
    RemoveLeadingZero(strftime("%d", localtime($start))) . ":" . 
    strftime("%H%M", localtime($start)) . ":" .
    strftime("%H%M", localtime($stop)) . ":$prio:" .
    "$lft:$name:$summary";
  my $found = 0;
  my $Update = 0;
  for(ParseTimer(1)) {
    if(($_->{cnumber} == $cnumber) && 
      ($_->{dor} == strftime("%d", localtime($start)))) {
      if(($_->{start} == strftime("%H%M", localtime($start))) 
        && ($_->{stop} == strftime("%H%M", localtime($stop)))) {
	$found = 1;
	if(($_->{name} ne $name) || ($_->{summary} ne $summary)) {
	  if($_->{active}) {
	    $Update = $_->{id};
	  }
	}
      }
    }
  }
  
  if($Update) {
    Log("Auto Timer: Updating Timer \"$newt\"");
    my($tmp) = SendCMD(1, "modt $Update $newt");
    if($tmp !~ /^\d+ $newt/) {
      Log("Auto Timer: Error updating Timer: \"$tmp\"");
    }
  }
  if(!$found) {
    Log("Auto Timer: Programming Timer \"$newt\"");
    my($tmp) = SendCMD(1, "newt $newt");
    if($tmp !~ /^\d+ $newt/) {
      Log("Auto Timer: Error programming Timer: \"$tmp\"");
    }
  }
  
}


##
#
##
sub ParseTimer {
  my $pc = shift;
  my $tid = shift;
  
  my(%chandesc) = GetChannelDesc();
  my @temp;
  for(SendCMD($pc, "lstt")) {
    last if(/^No timers defined/);
    chomp;
    my($id, $temp) = split(/ /, $_, 2);
    my($active, $cnumber, $dor, $start, $stop, $prio, $lft, $name, 
      $summary) = split(/\:/, $temp, 9);
    
    my($startsse);
    if(length($dor) == 7) {
      $startsse = timelocal(undef, substr($start, 2, 2), 
	substr($start, 0, 2), strftime("%d", localtime(time)), 
	(strftime("%m", localtime(time)) - 1),
	strftime("%Y", localtime(time)));
    } else {
      $startsse = timelocal(undef, substr($start, 2, 2), 
	substr($start, 0, 2), $dor, (strftime("%m", localtime(time)) - 1),
	strftime("%Y", localtime(time)));
    }
    
    my($endsse) = timelocal(undef, substr($stop, 2, 2), 
      substr($stop, 0, 2), strftime("%d", localtime(time)),
      (strftime("%m", localtime(time)) - 1),
      strftime("%Y", localtime(time)));
    
    my($sse) = $startsse;
    if(strftime("%d", localtime($sse)) < strftime("%d", localtime(time))) {
      $sse += 2678400;
    }
    push(@temp, {
      id     => $id,
      sse    => $sse,
      active => $active,
      cnumber=> $cnumber,
      cdesc  => $chandesc{$cnumber},
      dor    => $dor,
      start  => $start,
      stop   => $stop,
      prio   => $prio,
      lft    => $lft,
      name   => $name,
      summary=> $summary,
    });
  }
  
  if($tid) {
    return($temp[$tid-1]);
  } else {
    return(@temp);
  }
}


#############################################################################
#############################################################################
##
#
##
sub prog_detail {
  return if(UptoDate());
  my($titel, $text);
  for(@{$EPG{$q->param('chan')}}) {
    if($_->{id} == $q->param('epgid')) {
      $titel = $_->{progname} . ' | ' .
	strftime("%H:%M", localtime($_->{time}))
	. '-' . strftime("%H:%M", localtime($_->{time} + 
	$_->{duration})) . ' | ' . $_->{title};
      $text = $_->{description};
      last;
    }
  }
  my($template) = HTML::Template->new(filename => 
    "$TEMPLATEDIR/$CONFIG{LANG}/prog_detail.html");
  $template->param(
    titel => $titel ? $titel : undef, 
    text  => $text ? $text : undef,
  );
  SendHeader("200", "text/html", $template->output); 
}


##
#
##
sub prog_list {
  return if(UptoDate());
  if(!$q->param('number')) {
    SendForwardingHeader(
      "$MyURL?aktion=prog_list&number=" . $CHAN[0]->{number} );
    return;
  }  
  

  #
  my(@cn, $curchi); my($i) = 0;
  for(@CHAN) {
    ($curchi = $i) if($q->param('number') == $_->{number});
    $cn[$i] = $_->{number};
    $i++;
  } 
  my($nextch) = $cn[$curchi + 1];
  my($prevch) = $cn[$curchi - 1];

  
  my(@show, $progname, $cnumber); my($day) = 0;
  my($number) = $q->param('number');
  for my $content (@{$EPG{$number}}) {
    if(strftime("%d", localtime($content->{time})) != $day) {
      push(@show, {
	title  => $content->{progname} . " | " . 
	    FullDay(strftime("%w", localtime($content->{time}))) . ", "
	    . strftime("%d.%m.%Y", localtime($content->{time})),
	newd   => 1,
	nextch => $MyURL . "?aktion=prog_list&number=$nextch",
	prevch => $MyURL . "?aktion=prog_list&number=$prevch"
      });
      $day = strftime("%d", localtime($content->{time}));
    }
    push(@show, {
      ssse        => $content->{time},
      emit        => strftime("%H:%M", localtime($content->{time})),
      duration    => strftime("%H:%M", localtime($content->{time} + 
	$content->{duration})),
      title       => $content->{title},
      subtitle    => $content->{subtitle},
      recurl      => $MyURL . "?aktion=timer_new_form&active=1&chan="
	. $content->{number} . "&epgid=" . $content->{id} . "&zpuf=1&ref=" 
	. encode_ref($q),
      infurl      => $content->{description} ? $MyURL . 
	"?aktion=prog_detail&chan=".
	$content->{number} . "&epgid=" . $content->{id} : undef,
      newd        => 0,
    });
    $progname = $content->{progname};
    $cnumber = $content->{cnumber};
  }
  if($#show == -1) {
    push(@show, {
      title  => GetChannelDescByNumber($q->param("number")), 
      newd   => 1,
      nextch => $MyURL . "?aktion=prog_list&number=$nextch",
      prevch => $MyURL . "?aktion=prog_list&number=$prevch"
    });
  } 

  #
  my(@chans);
  for(@CHAN) {
    if($CONFIG{PCHAN_PL}) {
      my($f) = 0;
      for my $n (split(/\,/, $CONFIG{CHANS})) {
	($f = 1) if($n eq $_->{number});
      }
      next if(!$f);
    }
    push(@chans, {
      name => $_->{name},
      number => $_->{number},
      cur  => ($number == $_->{number}) ? 1 : 0,
    });
  }
  

  # 
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/prog_list.html",
    loop_context_vars => 1,
    die_on_bad_params => 0
  );
  $template->param(
    url      => $MyURL,
    loop     => \@show,
    chanloop => \@chans,
    progname => GetChannelDescByNumber($q->param("number")),
    switchurl  => $MyURL . "?aktion=prog_switch&channel=" . $cnumber
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub timer_list {
  return if(UptoDate()); 
    
  my @timer;
  for my $timer (ParseTimer(0)) {
    $timer->{delurl} = $MyURL . "?aktion=timer_delete&id=" . $timer->{id},
    $timer->{modurl} = $MyURL . "?aktion=timer_new_form&vdrid="
      . $timer->{id},
    $timer->{start} = substr($timer->{start}, 0, 2) . 
      ":" . substr($timer->{start}, 2, 2);
    $timer->{stop} = substr($timer->{stop}, 0, 2) . 
      ":" . substr($timer->{stop}, 2, 2);
    push(@timer, $timer);
  }
  @timer = sort({ $a->{sse} <=> $b->{sse} } @timer);
  
  #
  my(@at); 
  my($id) = 0;
  for(AT_Read()) {
    $id++;
    if($_->{start}) {
      $_->{start} = 
	substr($_->{start}, 0, 2) . ":" . substr($_->{start}, 2, 5);
    }
    if($_->{stop}) {
      $_->{stop} = 
	substr($_->{stop}, 0, 2) . ":" . substr($_->{stop}, 2, 5);
    }
    $_->{modurl} = $MyURL . "?aktion=at_timer_edit&id=$id";
    $_->{delurl} = $MyURL . "?aktion=at_timer_delete&id=$id";
    $_->{prio} = $_->{prio} ? $_->{prio} : $CONFIG{AT_PRIORITY};
    $_->{lft} = $_->{lft} ? $_->{lft} : $CONFIG{AT_LIFETIME};
    $_->{id} = $id;
    $_->{channel} = GetChannelDescByNumber($_->{channel}); 
    push(@at, $_);
  }
  @at = sort({ lc($a->{pattern}) cmp lc($b->{pattern}) } @at);
  
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/timer_list.html",
    die_on_bad_params => 0
  );
  $template->param(
    timer_loop => \@timer,
    at_timer_loop => \@at,
    at_timer_func => $CONFIG{AT_FUNC},
    nturl  => $MyURL . "?aktion=timer_new_form",
    naturl => $MyURL . "?aktion=at_timer_new&active=1",
    url    => $MyURL,
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub at_timer_edit {
  my $id = $q->param("id");

  my @at = AT_Read();
  
  #
  my @chans;
  for my $chan (@CHAN) {
    if($chan->{number}) {
      $chan->{cur} = ($chan->{number} == $at[$id-1]->{channel}) ? 1 : 0;
      push(@chans, $chan);
    }
  }
  
  my $template = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/at_new.html",
    die_on_bad_params => 0
  );
  $template->param(
    channels=> \@chans, 
    id      => $id,
    url     => $MyURL,
    prio    => $at[$id-1]->{prio} ? $at[$id-1]->{prio} : $CONFIG{AT_PRIORITY},
    lft     => $at[$id-1]->{lft} ? $at[$id-1]->{lft} : $CONFIG{AT_LIFETIME},
    active  => $at[$id-1]->{active},
    episode => $at[$id-1]->{episode},
    pattern => $at[$id-1]->{pattern},
    starth  => (length($at[$id-1]->{start}) >= 4) ? 
      substr($at[$id-1]->{start}, 0, 2) : undef,
    startm  => (length($at[$id-1]->{start}) >= 4) ?
      substr($at[$id-1]->{start}, 2, 5) : undef,
    stoph   => (length($at[$id-1]->{stop}) >= 4) ?
      substr($at[$id-1]->{stop}, 0, 2) : undef,
    stopm   => (length($at[$id-1]->{stop}) >= 4) ?
      substr($at[$id-1]->{stop}, 2, 5) : undef,
    title   => ($at[$id-1]->{section} & 1) ? 1 : 0,
    subtitle=> ($at[$id-1]->{section} & 2) ? 1 : 0,
    description => ($at[$id-1]->{section} & 4) ? 1 : 0,
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub at_timer_new {
  my $template = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/at_new.html",
    die_on_bad_params => 0
  );
  $template->param(
    url      => $MyURL, 
    active   => $q->param("active"),
    title    => 1,
    channels => \@CHAN,
    prio     => $CONFIG{AT_PRIORITY},
    lft      => $CONFIG{AT_LIFETIME},
  );
  SendHeader("200", "text/html", $template->output);
};


##
#
##
sub at_timer_save {
  my $id = $q->param("id");
  
  if($q->param("save")) {
    if(!$id) {
      my @at = AT_Read();
      my $section = 0;
      ($section += 1) if($q->param("title"));
      ($section += 2) if($q->param("subtitle"));
      ($section += 4) if($q->param("description"));
      push(@at, {
	episode  => $q->param("episode") ? $q->param("episode") : 0,
	active   => $q->param("active"),
	pattern  => $q->param("pattern"),
	section  => $section,
	start    => $q->param("starth") . $q->param("startm"),
	stop     => $q->param("stoph") . $q->param("stopm"),
	prio     => $q->param("prio"),
	lft      => $q->param("lft"),
	channel  => $q->param("channel"),
      });
      AT_Write(@at);
    } else {
      my $z = 0;
      my @at;
      for(AT_Read()) {
	$z++;
	if($z != $id) {
	  push(@at, $_);
	} else { 
	  my $section = 0;
	  ($section += 1) if($q->param("title"));
	  ($section += 2) if($q->param("subtitle"));
	  ($section += 4) if($q->param("description"));
	  push(@at, {
	    episode  => $q->param("episode") ? $q->param("episode") : 0,
	    active   => $q->param("active"),
	    pattern  => $q->param("pattern"),
	    section  => $section,
	    start    => $q->param("starth") . $q->param("startm"),
	    stop     => $q->param("stoph") . $q->param("stopm"),
	    prio     => $q->param("prio"),
	    lft      => $q->param("lft"),
	    channel  => $q->param("channel"),
	  });
	}
      }
      AT_Write(@at);
    }
    
    $CONFIG{LOCALLASTGET} = 0;
    UptoDate();
  }
  SendForwardingHeader("$MyURL?aktion=timer_list");
}


##
#
##
sub at_timer_delete {
  my $id = $q->param("id");
  my $z = 0;

  my @at = AT_Read();
  my @new;
  if($id) { 
    for(@at) {
      $z++;
      push(@new, $_) if($id != $z);
    }
    AT_Write(@new);
  } else {
    my @sorted;
    for($q->param) {
      if(/xxxx_(.*)/) {
	push(@sorted, $1);
      } 
    }
    my @sorted = sort({ $b <=> $a } @sorted);
    my $z = 0;
    for my $at (@at) {
      $z++;
      my $push = 1;
      for my $sorted (@sorted) {
	($push = 0) if($z == $sorted);
      }
      push(@new, $at) if($push);
    }
    AT_Write(@new); 
  }
  SendForwardingHeader("$MyURL?aktion=timer_list");
}


##
#
##
sub timer_new_form {
  UptoDate();

  my(@chans, $titel, $start, $stop, $cnumber, $summary, $title);
  my($prio, $lft, $active, $dor);
  if($q->param("epgid")) {
    for my $epg (@{$EPG{$q->param('chan')}}) {
      if($epg->{id} == $q->param('epgid')) {
	$start = $epg->{time};
	$stop = $epg->{time} + $epg->{duration};	
	$summary = $epg->{description};
	$title = $epg->{title};
	$dor = strftime("%d", localtime($epg->{time}));
	$active = 1;
	$cnumber = $epg->{number};
	last;
      }
    }
  } elsif($q->param("vdrid")) {
    my $hash = ParseTimer(0, $q->param("vdrid"));
    $title = $hash->{name};
    $summary = $hash->{summary};
    $active = $hash->{active};
    $dor = $hash->{dor};
    $prio = $hash->{prio};
    $lft = $hash->{lft};
    $cnumber = GetChannelID($hash->{cnumber});
    $start = timelocal(undef, substr($hash->{start}, 2, 2),
      substr($hash->{start}, 0, 2), $hash->{dor}, 
      strftime("%m", localtime(time))-1,
      strftime("%Y", localtime(time)));
    $stop = timelocal(undef, substr($hash->{stop}, 2, 2),
      substr($hash->{stop}, 0, 2), $hash->{dor}, 
      strftime("%m", localtime(time))-1,
      strftime("%Y", localtime(time)));
  } else {
    $start = timelocal(undef, 0, 1, strftime("%d", localtime(time)),
	(strftime("%m", localtime(time)) - 1),
	strftime("%Y", localtime(time)));
    $active = 1;
  }

  for my $chan (@CHAN) {
    push(@chans, {
      name => $chan->{name},
      id   => $chan->{id},
      cur  => $cnumber ?
	(($chan->{number} == $cnumber) ? 1 : 0) : undef,
    });
  }
  
  if($q->param('zpuf')) {
    $start -= ($CONFIG{ZPUFA} * 60);
    $stop += ($CONFIG{ZPUFE} * 60);
  }    
      
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/timer_new.html",
    die_on_bad_params => 0
  );
  $template->param(
    titel  => "Neuer Timer",
    url    => $MyURL,
    chans  => \@chans,
    active => $active ? $active : 0,
    dor    => (length($dor) == 7) ? $dor :
      strftime("%d", localtime($start)),
    starth => (strftime("%H", localtime($start)) ne "") ?
      strftime("%H", localtime($start)) : undef,
    startm => (strftime("%M", localtime($start)) ne "") ?
      strftime("%M", localtime($start)) : undef,
    endh   => (strftime("%H", localtime($stop)) ne "") ?
      strftime("%H", localtime($stop)) : undef,
    endm   => (strftime("%M", localtime($stop)) ne "") ?
      strftime("%M", localtime($stop)) : undef,
    prio   => $prio ? $prio : $CONFIG{PRIORITY},
    lft    => $lft ? $lft : $CONFIG{LIFETIME},
    title  => $title ? $title : undef,
    summary=> $summary ? $summary : undef,
    vdrid  => $q->param("vdrid") ? $q->param("vdrid") : 0,
    ref    => $q->param("ref") ? $q->param("ref") : undef
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub timer_add {
  my($start, $end, $prio, $lft, $title);
  
  if($q->param("save")) { 
    #
    my($dor) = $q->param('dor');
    
    #
    if($q->param('starth') =~ /\d+/ && $q->param('starth') < 24 && $q->param('starth') >= 0) {
      if($q->param('startm') =~ /\d+/ && $q->param('startm') < 60 && $q->param('starth') >= 0) {
	$start = $q->param('starth') . $q->param('startm');
      } else {
	HTMLError("Error: Bitte Startzeit richtig eingeben!");
      }
    } else { 
      HTMLError("Error: Bitte Startzeit richtig eingeben!");
    }
    
    #
    if($q->param('endh') =~ /\d+/ && $q->param('endh') < 24 && $q->param('endh') >= 0) {
      if($q->param('endm') =~ /\d+/ && $q->param('endm') < 60 && $q->param('endm') >= 0) {
	$end = $q->param('endh') . $q->param('endm');
      } else {
	HTMLError("Error: Bitte Endzeit richtig eingeben!");
      }
    } else { 
      HTMLError("Error: Bitte Endzeit richtig eingeben!");
    }

    
    #
    if($q->param('prio') =~ /\d\d/) {
      $prio = $q->param('prio');
    } else {
      HTMLError("Error: Bitte Prioritaet richtig eingeben!");
    }

    #
    if($q->param('lft') =~ /\d\d/) {
      $lft = $q->param('lft');
    } else {
      HTMLError("Error: Bitte Lifetime richtig eingeben!");
    }
    
    #
    if($q->param('title')) {
      $title = $q->param('title');
      $title =~ s/\://g;
    } else {
      HTMLError("Error: Bitte Titel eingeben!");
    }
    
    if(!$q->param('vdrid')) {
      SendCMD(0, "newt " . $q->param('active') . ":" . $q->param('chans')
	. ":" . $dor . ":" . $start . ":" . $end  . ":" . $prio . ":"
	. $lft . ":" . $title . ":" . $q->param('summary'));
    } else {
      SendCMD(0, "modt " . $q->param('vdrid') . " " . $q->param('active')
	. ":" . $q->param('chans') . ":" . $dor . ":" . $start . ":"
	. $end  . ":" . $prio . ":" . $lft . ":" . $title . ":" .
	$q->param('summary'));
    }
  }
  if(!$q->param("ref")) {
    SendForwardingHeader("$MyURL?aktion=timer_list");
  } else {
    SendForwardingHeader("$MyURL?" . decode_ref($q->param("ref")));
  }
}


##
#
##
sub timer_delete {
  my($id) = $q->param('id');
  if($id) {
    my($result) = SendCMD(0, "delt $id");
    if($result =~ /Timer "$id" is recording/i) {
      SendCMD(0, "modt $id off");
    }
  } else {
    my @sorted;
    for($q->param) {
      if(/xxxx_(.*)/) {
	push(@sorted, $1);
      }
    }
    my @sorted = sort({ $b <=> $a } @sorted);
    for my $t (@sorted) {
      my($result) = SendCMD(1, "delt $t");
      if($result =~ /Timer "$t" is recording/i) {
	SendCMD(1, "modt $t off");
      }
    }
    CloseSocket();
  }
  SendForwardingHeader("$MyURL?aktion=timer_list");
}


##
#
##
sub prog_summary {
  return if(UptoDate());

  my($search) = $q->param('search');
  my($eventtime);
  
  if($q->param('time') ne "") {
    my($hour, $minute);
    if($q->param('time') =~ /(\d{1,2})(\D?)(\d{1,2})/) {
      if(length($1) == 1 && length($3) == 1 && !$2) {
	$hour = $1 . $3;
      } else {
	($hour, $minute) = ($1, $3);
      }  
    } elsif($q->param('time') =~ /\d/) {
      $hour = $q->param('time');
    }
    
    if($hour <= strftime("%H", localtime(time)) && $minute < strftime("%M", localtime(time))) {
      $eventtime = timelocal(undef, $minute, $hour, strftime("%d", 
	localtime(time + 86400)), 
	(strftime("%m", localtime(time + 86400)) - 1), 
	strftime("%Y", localtime(time))) + 1;
    } else {
      $eventtime = timelocal(undef, $minute, $hour, strftime("%d", 
	localtime(time)), (strftime("%m", localtime(time)) - 1), 
	strftime("%Y", localtime(time))) + 1;
    } 
  } else {
    $eventtime = time;
  }
	  
  #
  my @chans;
  for(@CHAN) {
    (my($radio) = 1) if(!$_->{vpid});
    push(@chans, {
      name   => $_->{name},
      radio  => $radio,
      url    => $MyURL . "?aktion=prog_list&number=" . $_->{number},
      number => $_->{number}
    });
  } 
  
  my(@temp);
  for(keys(%EPG)) {
    for(@{$EPG{$_}}) {
      if(!$search) {
	if($CONFIG{PCHANL}) {
	  my($f) = 0;
	  for my $n (split(/\,/, $CONFIG{CHANS})) {
	    ($f = 1) if($n eq $_->{number});
	  }
	  next if(!$f);
	}
	next if($eventtime > ($_->{time} + $_->{duration}));
      } else {
	my($found);
	for my $word (split(/ +/, $search)) {
	  $found = 0;
	  for my $section (qw(title subtitle description)) {
	    if($_->{$section} =~ /$word/i) {
	      $found = 1;
	    }
	  }
	  if(!$found) {
	    last;
	  }
	}
	next if(!$found);
      }
      push(@temp,  {
	sse        => $_->{time},
	prog       => $_->{progname},
	date       => strftime("%d.%m.%Y", localtime($_->{time})),
	emit       => strftime("%H:%M", localtime($_->{time})),
	duration   => strftime("%H:%M", localtime($_->{time} +
	  $_->{duration})),
	title      => $_->{title} ? $_->{title} : undef,
	subtitle   => $_->{subtitle} ? ((length($_->{subtitle}) > 30) ? 
	  substr($_->{subtitle}, 0, 30) . "..." : $_->{subtitle}) : undef,
	summary    => $_->{description} ? substr($_->{description}, 0, 120)
	  . "... " : undef,
	infurl     => $_->{description} ? $MyURL . 
	  "?aktion=prog_detail&chan=" . $_->{number} . "&epgid=" . $_->{id}
	  : undef,
	recurl     => $MyURL . "?aktion=timer_new_form&active=1&chan="
	    . $_->{number} . "&epgid=" . $_->{id} . "&zpuf=1&ref=" 
	    . encode_ref($q),
	proglink   => $MyURL . "?aktion=prog_list&number=" . $_->{number},
	row0       => undef,
	row1       => undef,
	cnumber    => $_->{cnumber},
	switchurl  => $MyURL . "?aktion=prog_switch&channel=" . 
	  $_->{cnumber},
      });
      last if(!$search);
    }
  }
  my($rows); my($maxrows) = 2; my(@show); my($toggle) = 0; 
  for(sort({ $a->{cnumber} <=> $b->{cnumber} } @temp)) {
    ($rows == 0) ? ($_->{row0} = 1) : ($_->{row0} = 0);
    ($rows == $maxrows) ? ($_->{row1} = 1) : ($_->{row1} = 0);
    push(@show, $_);
    if($rows < $maxrows) {
      $rows++;
    } else {
      $rows = 0;
    }
  }
  if($rows) {
    for(; $rows <= $maxrows; $rows++) {
      if($rows == $maxrows) {	
	push(@show, { row1 => 1 });
      } else {
	push(@show, { row0 => undef });
      }
    }
  }
 
  # 
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/prog_summary.html",
    die_on_bad_params => 0
  );
  $template->param(
    rows   => \@show,
    now    => strftime("%H:%M", localtime($eventtime)),
    nowurl => $MyURL . "?aktion=prog_summary",
    url    => $MyURL
  );
  SendHeader("200", "text/html", $template->output);
}


#
sub rec_list {	
  my(@temp);
  my($sortby) = $q->param('sortby');
  my(@response) = SendCMD(0, "lstr");
  for(@response) {
    chomp;
    next if(length($_) == 0);
    if(/^Command unrecognized\: \"lstr\"/) {
      HTMLError("Recording's listing not possible, please apply the"
	. "patch 'svdrp-0.xy.patch' delivered with this package!");
      return;
    }
    if(/^No recordings available/) {
      last;
    }
    my($new);
    my($id, $date, $time, $name) = split(/ +/, $_, 4);
    
    #
    #$name =~ s/\~/ - /;
    
    #
    if(substr($time, 5, 1) eq "*") {
      $new = 1;
      $time =~ s/\*//;
    }

    push(@temp, {
      sse    => timelocal(undef, substr($time, 3, 2), 
	substr($time, 0, 2), substr($date, 0, 2),
	(substr($date, 3, 2) - 1),
	strftime("%Y", localtime(time))),
      date   => $date,
      time   => $time,
      name   => $name,
      new    => $new,
      id     => $id,
      delurl => $MyURL . "?aktion=rec_delete&id=$id",
      infurl => $MyURL . "?aktion=rec_detail&id=$id"
    });
  }
  my(@recordings);
  if($sortby eq "time") {
    @recordings = sort({ $a->{time} cmp $b->{time} } @temp);
  } elsif($sortby eq "name") {
    @recordings = sort({ lc($a->{name}) cmp lc($b->{name}) } @temp);
  } else {
    @recordings = sort({ $a->{sse} cmp $b->{sse} } @temp);
  }

  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/rec_list.html",
    die_on_bad_params => 0
  );
  $template->param(
    recloop => \@recordings,
    sortbydate => $MyURL . "?aktion=rec_list",
    sortbytime => $MyURL . "?aktion=rec_list&sortby=time",
    sortbyname => $MyURL . "?aktion=rec_list&sortby=name",
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub rec_delete {
  my($id) = $q->param('id');
  if($id) {
    SendCMD(1, "delr $id");
  } else {
    for($q->param) {
      if(/xxxx_(.*)/) {
	SendCMD(1, "delr $1");
      }
    }
  }
  CloseSocket();
  SendForwardingHeader("$MyURL?aktion=rec_list");
}


##
#
##
sub show_top {
  my $template = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/top.html",
    die_on_bad_params => 0
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub rec_detail {
  my($id) = $q->param('id');

  #
  my($i, $titel);
  for(SendCMD(0, "lstr")) {
    ($i, undef, undef, $titel) = split(/ +/, $_, 4);
    last if($id == $i);
  }
  chomp($titel);

  #
  my($text); my($first) = 1;
  my($result) = SendCMD(0, "lstr $id");
  if($result !~ /No summary availab/i) {
    for(split(/\|/, $result)) {
      if($_ ne (split(/\~/, $titel))[1]) {
	if($first && $titel !~ /\~/ && length($titel) < 20) {
	  $titel .= "~" . $_;
	  $first = 0;
	} else {
	  $text .= $_;
	}
      } 
    }
  }

  #
  $titel =~ s/\~/ - /;
      
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/prog_detail.html",
    die_on_bad_params => 0
  );

  $template->param(
    text  => $text ? $text : "",
    titel => $titel
  );
  SendHeader("200", "text/html", $template->output);	
}


##
#
##
sub conf_list {  
  return if(UptoDate());
  
  my(@lang, @allchans, @selchans, @temp);
  
  if($q->param('submit') eq ">>>>>") {
    for my $c ($q->param('allchans')) {
      my($found) = 0;
      for(split(/\,/, $CONFIG{CHANS})) {
	if($_ eq $c) {
	    $found = 1;
	}
      }
      if(!$found) {
	if($CONFIG{CHANS}) {
	    $CONFIG{CHANS} .= "," . $c;
	} else {
	    $CONFIG{CHANS} = $c;
	}
      }
    }
    $CONFIG{PCHANL} = 1;
  }
  if($q->param('submit') eq "<<<<<") {
    if($CONFIG{CHANS}) {
      for my $c ($q->param('selchans')) {
	my($temp);
	for(split(/\,/, $CONFIG{CHANS})) {
	  if($_ ne $c) {
	    if($temp) {
	      $temp .= ",$_";
	    } else {
	      $temp = $_;
	    }
	  }
	}
	$CONFIG{CHANS} = $temp;
      }
    }
    $CONFIG{PCHANL} = 1;
  }
  if($q->param('save')) {
    for($q->param) {
      if(/[A-Z]+/) {
	$CONFIG{$_} = $q->param($_);
      }
    }
    open(CONF, ">$CONFFILE");
    for(keys(%CONFIG)) {
      print(CONF "$_ = $CONFIG{$_}\n");
    }
    close(CONF);
    
    #
    $Socket->timeout($CONFIG{AT_TIMEOUT} * 60) if($CONFIG{AT_FUNC});
  }
  if($q->param('apply')) {
    for($q->param) {
      if(/[A-Z]+/) {
	$CONFIG{$_} = $q->param($_);
      }
    }
    
    #
    $Socket->timeout($CONFIG{AT_TIMEOUT} * 60) if($CONFIG{AT_FUNC});
  }
      
  
  #	
  for my $dir (<$TEMPLATEDIR/*>) {
    next if(!-d $dir);
    $dir =~ s/.*\///g;
    my($found) = 0;
    for(@lang) { ($found = 1) if($_->{name} eq $1); }
    if(!$found) { 
      push(@lang, {
	name   => $dir,
	aklang => ($CONFIG{LANG} eq $dir) ? 1 : 0,
      }); 
    }
  }
  

  #
  for my $c (@CHAN) {
    push(@allchans, {
      name => $c->{name},
      number => $c->{number}
    });
    my($found) = 0;
    for(split(/\,/, $CONFIG{CHANS})) {
      if($_ eq $c->{number}) {
	$found = 1;
      }
    }
    next if(!$found); 
    push(@selchans, {
      name => $c->{name},
      number => $c->{number}
    }); 
  }
  
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/config.html",
    die_on_bad_params => 0
  );
  $template->param(
    %CONFIG,
    LANGLIST      => \@lang,
    ALLCHANS      => \@allchans,
    SELCHANS      => \@selchans,
    url           => $MyURL,
  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub prog_switch {
  my $channel = $q->param("channel");
  if($channel) {
    SendCMD(0, "chan $channel");
  }
  SendFile($BASENAME . "/images/switch_channel.gif");    
}


##
#
##
sub rc_show {
  my($template) = HTML::Template->new(
    filename => "$TEMPLATEDIR/$CONFIG{LANG}/rc.html",
    die_on_bad_params => 0
  );
  $template->param(
    surl_0         => $MyURL . "?aktion=rc_hitk&key=0",
    surl_1         => $MyURL . "?aktion=rc_hitk&key=1",
    surl_2         => $MyURL . "?aktion=rc_hitk&key=2",
    surl_3         => $MyURL . "?aktion=rc_hitk&key=3",
    surl_4         => $MyURL . "?aktion=rc_hitk&key=4",
    surl_5         => $MyURL . "?aktion=rc_hitk&key=5",
    surl_6         => $MyURL . "?aktion=rc_hitk&key=6",
    surl_7         => $MyURL . "?aktion=rc_hitk&key=7",
    surl_8         => $MyURL . "?aktion=rc_hitk&key=8",
    surl_9         => $MyURL . "?aktion=rc_hitk&key=9",
    
    surl_power     => $MyURL . "?aktion=rc_hitk&key=Power",
    
    surl_ok        => $MyURL . "?aktion=rc_hitk&key=Ok",

    surl_menu      => $MyURL . "?aktion=rc_hitk&key=Menu",
    surl_back      => $MyURL . "?aktion=rc_hitk&key=Back",
    
    surl_up        => $MyURL . "?aktion=rc_hitk&key=Up",
    surl_down      => $MyURL . "?aktion=rc_hitk&key=Down",
    surl_left      => $MyURL . "?aktion=rc_hitk&key=Left",
    surl_right     => $MyURL . "?aktion=rc_hitk&key=Right",

    surl_red       => $MyURL . "?aktion=rc_hitk&key=Red",
    surl_green     => $MyURL . "?aktion=rc_hitk&key=Green",
    surl_blue      => $MyURL . "?aktion=rc_hitk&key=Blue",
    surl_yellow    => $MyURL . "?aktion=rc_hitk&key=Yellow",
    
    surl_volplus   => $MyURL . "?aktion=rc_hitk&key=Volume%2B",
    surl_volminus  => $MyURL . "?aktion=rc_hitk&key=Volume-",

  );
  SendHeader("200", "text/html", $template->output);
}


##
#
##
sub rc_hitk {
  my $key = $q->param("key");
  SendCMD(0, "hitk $key");
  SendFile("images/transe.gif");
  #SendHeader("200", "text/html", "0");
}

# EOF
